' Simple terminal debug program a biy like FTP, you type a commnad
' followed by a parameter. Commands can be abbreviated to (usually)
' the first two letters.
' SEND - abbreviation SE (text to send)
'    Sends the text following the command to the comm port
'    example SE FRED
'    will send FRED to the comm port
'
' DEBUG - abbreviation DE
'    Gets contents of input buffer from comm port and
'    prints out in debug format. The format is hex bytes
'    followed by their ASCII representation
'
' ?
'    Prints a summary of commands
'
' QUIT - abbreviation QU
'    Quits and closes the comm port
'
' COMPORT - abbreviation CO (comm. port #)
'    Change comm port number, e.g. COM 3
'
' PAUSE - abbreviation PA (milliseconds)
'    Halts program execution for specified number
'    of milliseconds
'
' WAIT - abbreviation WA (byte value)
'    Waits at the comm port input buffer for the specified
'    value and returns when received. If not received after
'    200 iterations it will print ACK error
'    Example WA 62
'    This will wait for the > of the L>
'
' FILE - abbreviation FI (file name)
'    FI without a file name will bring up a dialogue box.
'    the file contents will be used as the input to the program
'    Good for sending a series of commands
' --------------------------------------------------------------
' start up
' --------------------------------------------------------------
    global finished
    global filepause$
    call  doComm "1"        ' default com port
    filepause$ = "50"       ' pause when using file input
    print "For help ?"
'
' --------------------------------------------------------------
' simple main loop
' --------------------------------------------------------------
[loop]
    input a$
    call CommandEr a$
    if finished = 1 then [quit]
    goto [loop]
'
' --------------------------------------------------------------
' command parser, determines command from the first word and then
' sends it to the correct sub along with anything following
' the command. If a command is not recocnised it will send the
' text in inline$ dorectly to the comm port
' --------------------------------------------------------------
sub CommandEr inline$
    finished = 0
    token$ = upper$(word$(inline$, 1)) ' get command
    restOFline$ = mid$(inline$, len(token$)+2) ' and rest of line
    token$ = left$(token$, 2) ' only first 2 letters significant
    ' now act upon a series of upper case 2 letter commands
    select case token$
        case "?"
            call doHelp
        case "QU"
            finished = 1            ' tells calling to quit
        case "SE"
            call doSend restOFline$
        case "DE"
            call doDebug restOFline$
        case "CO"
            call doCommChange restOFline$
        case "PA"
            call doPause restOFline$
        case "WA"
            call doWait restOFline$
        case "FI"
            call doSendFile restOFline$ 
        case else
            if len(inline$) > 0 then
                print #comm, inline$
            else
                print #comm, chr$(13);  ' required for initialising
            end if
            print CommIn$()
    end select
end sub
'
' --------------------------------------------------------------
' Help simply prints a list of commands
' --------------------------------------------------------------
sub doHelp
    print
    print "Commmands can be abreviate to 2 letters"
    print "? This text"
    print "QUIT     to end program -- QU"
    print "SEND     to send string to comm port -- SE"
    print "DEBUG    to receiev a string from comm port  -- DE"
    print "COMM     change com port number -- CO"
    print "PAUSE    for x number of milliseconds -- PA x"
    print "WAIT     wait for x to be received -- WA x"
    print "FILE     run file x -- FI x"
end sub
'
' --------------------------------------------------------------
' Sends string following the 'SEND' command to the com port
' --------------------------------------------------------------
sub doSend tline$
    if len(tline$) > 0 then
       print #comm, tline$
    else
       print #comm, chr$(13);  ' required for initialising
    end if
end sub
'
' --------------------------------------------------------------
' debug print. Prints contents of comm input buffer in debug
' format.
' e.g.  00 01 41 42 43 : ..ABC
' uses hex notation, ASCII is after the :
' --------------------------------------------------------------
sub doDebug tline$
    c$ = input$(#comm, lof(#comm))
    lastReceived$ = c$
    for j = 1 to len(c$)
        a = asc(mid$(c$,j))  ' get value
        call PrnHexByte a    ' print in hex
        print " ";           ' and a space
    next j
    print ":";
    call PrnAscii c$
end sub
'
' --------------------------------------------------------------
' Open comm port
' --------------------------------------------------------------
sub doComm  portnum$
    'Set the size of the communications buffer to 16K
    Com = 8192 * 2
    cpt$ = "COM"+portnum$+":9600,n,8,2,ds0,cs0,rs"
    open cpt$ for random as #comm
end sub
'
' --------------------------------------------------------------
' changes comm port
' arrive here with the rest of the input line that should be a
' number
' --------------------------------------------------------------
sub doCommChange newport$
    close #comm
    call doComm newport$
end sub
'
' --------------------------------------------------------------
' pauses here for x number of milliseconds
' --------------------------------------------------------------
sub doPause mil$
    mil = val(mil$)
    t=time$("milliseconds")
    while time$("milliseconds")<t+mil
    scan
    wend
end sub
'
' --------------------------------------------------------------
' waits for comm buffer and looks for a specified character
' if found simply returns, if not prints error after waiting
' a number of iterations. Use this to wait for a response from
' external device ** Cant do zeroes **
' --------------------------------------------------------------
sub doWait value$
    v = val(value$)
    err = 200
    for j = 1 to err
        if v = asc(input$(#comm,1)) then exit for
    next
    if j > err then print "ACK Error"
end sub
'
' --------------------------------------------------------------
' sends a file to the command processor, in other words takes
' over this input until end of file
' --------------------------------------------------------------
sub doSendFile filename$
    if len(filename$) < 1 then
        filedialog "Open..." , "*.txt", filename$
    end if
    if len(filename$) > 0 then
       open filename$ for input as #text
       while eof(#text) = 0
           line input #text, item$
           call CommandEr item$ 
           call doPause filepause$
       wend
    end if
    close #text
end sub
'
' ==============================================================
' uttility functions below here
'
'
' --------------------------------------------------------------
' a function that will return the ascii code for a given
' hex value from 0 to 15
' --------------------------------------------------------------
function ConvNibble(value)
    if value < 10 then
        ConvNibble = value + 48
    else
        ConvNibble = value + 55
    end if
end function
'
' --------------------------------------------------------------
' prints a hex BYTE value to 2 places
' put in-line for simplicity
' --------------------------------------------------------------
sub PrnHexByte a
    b = ConvNibble(int( a / 16))
    print chr$(b);
    b = ConvNibble((a mod 16))
    print chr$(b);
end sub
'
' --------------------------------------------------------------
' delayed input, allow comm port time to respond
' use this just after outputting a command
' --------------------------------------------------------------
function CommIn$()
    for j = 1 to 300
        if lof(#comm) <> 0 then CommIn$ = CommIn$ + input$(#comm, 1)
    next j
    lastReceived$ = CommIn$
end function
'
' --------------------------------------------------------------
' given a text input will print out an ascii text representation
' as 1 line, any non-prinable characters <32 >127 will be
' printed as a .
' --------------------------------------------------------------
sub PrnAscii text$
    for j = 1 to len(text$)
        a = asc(mid$(text$,j))
        if (a<32) or (a>127) then
            print ".";
        else
            print chr$(a);
        end if
    next j
    print
end sub
'

[quit]
    close #comm
    end
